#include "StdAfx.h"
#include "CGAL_Object.h"


CGAL_Object::CGAL_Object(void) 
: m_bbox( BBox_2(FLOAT_MAX, FLOAT_MAX, -FLOAT_MAX, -FLOAT_MAX) )
, m_center( Point_2(0, 0) )
, m_fFitScale( 1 )
, m_nDisplayListObject( 0 )
{}


CGAL_Object::~CGAL_Object(void)
{}


void CGAL_Object::ComputeGeometryParameters(const std::list<std::list<Point_2>>& polygons)
{
	std::list< std::list<Point_2> >::const_iterator polygon_cur = polygons.begin();
	std::list< std::list<Point_2> >::const_iterator polygon_end = polygons.end();

	m_bbox = CGAL::bbox_2(polygon_cur->begin(), polygon_cur->end());
	for (++polygon_cur; polygon_cur!=polygon_end; ++polygon_cur)
		m_bbox = m_bbox + CGAL::bbox_2(polygon_cur->begin(), polygon_cur->end());

	double cx = ( m_bbox.xmax() + m_bbox.xmin() ) / 2.0;
	double cy = ( m_bbox.ymax() + m_bbox.ymin() ) / 2.0; 
	m_center = Point_2(cx, cy);

	// Calculate proper rendering position
	double len = max( m_bbox.xmax() - m_bbox.xmin(), m_bbox.ymax() - m_bbox.ymin() );
	m_fFitScale = 1.5 / len;
}
