/*
* Copyright (c) 2007, Shenyang Institute of Automation, Chinese Academy
* of Sciences. , all rights reserved.
* 
* Filename     : MeshSegmentation.h
* Project	   : xMesh library
* Abstract     : Declaration of the MeshSegmentation class
* Version      : 1.0
* Author			: , nanlliang@sia.cn
* Original Author   : , nanlliang@sia.cn
* Accomplished date : Dec. 29, 2007
*/

//-----------------------------------------------------------------


#ifndef _MESH_SEGMENTATION_H_
#define _MESH_SEGMENTATION_H_

#include "TriMesh.h"

#include <set>
#include <map>
#include <vector>
#include <gl/GL.h>



enum  FeatureCriterion {FC_RATIO, FC_DIHEDRAL};

class MeshSegmentation
{
public:
	MeshSegmentation(void);
	~MeshSegmentation(void);

public:
	void SetTarget(TriMesh* pMesh);
	void Clear();
	void Apply(FeatureCriterion criterion, float value);
	void Draw();
	void SetShowSegmentation(bool b);


private:
	void DetectFeatureEdges(FeatureCriterion criterion, float value);

	void EnumerateComponents(std::vector<int>&  faceID) ;

	void PropagateComponent(const TriFace& face, 
							std::map<TriFace, int>& faceIndices, 
							const std::set<Edge3D>&	featureEdges, 
							int cur_ID) ;

	// 淨ڻ
	double FaceNormalProduct(const Edge3D& edge) const;

	// 淨н
	double FaceNormalAngle(const Edge3D& edge) const;

	void   BuildSegmentationList();	

private:
	TriMesh*			 m_pMesh;
	std::vector<int>	 m_faceID;
	int                  m_nNumComponents;

	GLuint				 m_listSegmentation;
	bool				 m_bShowSegmentation;
};


#endif //_MESH_SEGMENTATION_H_