/*
* Copyright (c) 2007, Shenyang Institute of Automation, Chinese Academy
* of Sciences. , all rights reserved.
* 
* Filename     : OpenGLView.cpp
* Project	   : xMesh library
* Abstract     : Implementation file of COpenGLView class
* Version      : 1.0
* Author			: , nanlliang@sia.cn
* Original Author   : , nanlliang@sia.cn
* Accomplished date : Dec. 29, 2007
*/


#include "stdafx.h"
#include "OpenGLView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COpenGLView

IMPLEMENT_DYNCREATE(COpenGLView, CView)

COpenGLView::COpenGLView()
{
	m_hRC = NULL;
	m_hDC = NULL;
	m_pDC = NULL;
	
	InitGeometry();
}


COpenGLView::~COpenGLView()
{
}


BEGIN_MESSAGE_MAP(COpenGLView, CView)
//{{AFX_MSG_MAP(COpenGLView)
ON_WM_CREATE()
ON_WM_SIZE()
ON_WM_DESTROY()	
ON_WM_ERASEBKGND()
ON_WM_LBUTTONDOWN()
ON_WM_LBUTTONUP()
ON_WM_MOUSEMOVE()
ON_WM_RBUTTONDOWN()
ON_WM_RBUTTONUP()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// COpenGLView diagnostics

#ifdef _DEBUG
void COpenGLView::AssertValid() const
{	CView::AssertValid();}

void COpenGLView::Dump(CDumpContext& dc) const
{	CView::Dump(dc);}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// COpenGLView drawing
void COpenGLView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	
	// you will always want to redefine this in your subclass
	// Here we assume that the one and only rendering context is selected
	// If it for some reason is not, this must be
	// done before we do any OpenGL-action

	// Clear the color and depth buffers.
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Set up for scene
	glMatrixMode(GL_MODELVIEW);
 	glLoadIdentity();	
	
	glPushMatrix();			
		glTranslatef(m_xTranslation, m_yTranslation, m_zTranslation);		
		glRotatef(m_xRotation, 1.0, 0.0, 0.0);      
		glRotatef(m_yRotation, 0.0, 1.0, 0.0);
		glRotatef(m_zRotation, 0.0, 0.0, 1.0);	
		glScalef(m_Scaling, m_Scaling, m_Scaling);	

		RenderSceneScalable();
	glPopMatrix();

	glPushMatrix();
		if (m_bShowAxises)      
		{
			glRotatef(m_xRotation, 1.0, 0.0, 0.0);  
			glRotatef(m_yRotation, 0.0, 1.0, 0.0);
			glRotatef(m_zRotation, 0.0, 0.0, 1.0);	

			GLboolean bLightOn = glIsEnabled(GL_LIGHTING);
			if (bLightOn)
				glDisable(GL_LIGHTING);  
			GLboolean bCullFace = glIsEnabled(GL_CULL_FACE);
			if (bCullFace)
				glDisable(GL_CULL_FACE);

			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL); 
			RenderSceneUnScalable();    

			if (bLightOn)
				glEnable(GL_LIGHTING);   
			if (bCullFace)
				glEnable(GL_CULL_FACE);
		}
	glPopMatrix();

		
	// finish drawing
	glFlush();
	SwapBuffers(wglGetCurrentDC());	
}


void COpenGLView::RenderSceneScalable()
{
	glutSolidSphere(1.5f, 20, 20);
}


void COpenGLView::RenderSceneUnScalable()
{
	glCallList(m_nAxisesList);
}

/////////////////////////////////////////////////////////////////////////////
// COpenGLView message handlers
int COpenGLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	// sets up the drawing environment and do
	// one time initialization of OpenGL-settings
	
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
    m_pDC = new CClientDC(this);
	m_hDC = m_pDC->GetSafeHdc();
	ASSERT(m_hDC != NULL);
	
	// Set up the drawing environment
	SetUpDrawingEnvironment();
	ASSERT(m_hRC != NULL);
	
	// do one-time initializing
	InitializeRContext();

	PrepareAxisLabel();
	BuildAxisesList();

	return 0;
}


void COpenGLView::OnDestroy() 
{	
	CView::OnDestroy();	
	// TODO: Add your message handler code here
	// only one dynamic variable to release
	
	if(m_pDC != NULL)
		delete m_pDC;
}


void COpenGLView::OnSize(UINT nType, int cx, int cy) 
{	
	CView::OnSize(nType, cx, cy);

	glViewport(0, 0, cx, cy);
	
    // We will work on the projection matrix,
	// so set matrix mode
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	
	// set up your viewing volume
	float aspect = (cy == 0) ? cx : cx / (float)cy;

	glOrtho(m_OrthoRangeLeft * aspect, m_OrthoRangeRight * aspect,
		m_OrthoRangeBottom, m_OrthoRangeTop,
		m_OrthoRangeNear, m_OrthoRangeFar);	
	
	// set back matrix mode
	glMatrixMode(GL_MODELVIEW);	
	glLoadIdentity();
}


BOOL COpenGLView::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}


BOOL COpenGLView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class
	// all OpenGL windows must have this window-style
	
	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	return CView::PreCreateWindow(cs);
}


void COpenGLView::InitGeometry()
{
	m_bEnableLight = true;
	m_bShowAxises = false;
	m_bLeftButtonDown = false;
	m_bRightButtonDown = false;
	
	m_nAxisesList = true;

	m_xRotation = 0.0;
	m_yRotation = 0.0;
	m_zRotation = 0.0;
	
	m_xTranslation = 0.0f;
	m_yTranslation = 0.0f;
	m_zTranslation = 0.0f;	
	
	m_Scaling = 1.0f;
	
	m_SpeedScaling = 1.05;
	m_SpeedRotation = 1.0f / 3.0f;
	m_SpeedTranslation = 1.0f / 212.0f;
	
	// ת;
	m_StepRotationX = 0.0f;
	m_StepRotationY = 5.0f;
	m_StepRotationZ = 0.0f;
	
	// ɫ;
	m_BackColorRed = 0.0f;
	m_BackColorGreen = 0.0f;
	m_BackColorBlue = 0.0f;
	m_BackAlpha = 1.0f;

	//ƽͶӰ
	m_OrthoRangeLeft = -1.5f;
	m_OrthoRangeRight = 1.5f;
	m_OrthoRangeBottom = -1.5f;
	m_OrthoRangeTop = 1.5f;
	m_OrthoRangeNear = -50.0f;
	m_OrthoRangeFar = 50.0f;
}


//////////////////////////////////////////////////
// default one-time initialization
void COpenGLView::InitializeRContext()
{
	// Here we set the attributes that probably
	// will be constant during the rendering 
	// contexts lifetime.
	
	// you may or may not want this function
	// if you don't need it, simply redefine an empty
	// function in your subclass

    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);

	// prepare light source
	GLfloat lightAmbient[] = {0.45f, 0.45f, 0.45f, 1.0f};
	GLfloat lightDiffuse[] = {1.0f, 1.0f, 1.0f, 1.0f};
	GLfloat lightSpecular[] = {0.6f, 0.6f, 0.6f, 1.0f};
 	GLfloat lightPosition[] = {30.0f, 30.0f, 50.0f, 1.0f};
 	
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
 	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
 	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);	

	GLfloat materialAmbient[] = {0.329412f, 0.223529f, 0.027451f, 1.0f};
	GLfloat materialDiffuse[] = {0.780392f, 0.568627f, 0.113725f, 1.0f};
	GLfloat materialSpecular[] = {0.992157f, 0.941176f, 0.807843f, 1.0f};
	glMaterialfv(GL_FRONT, GL_AMBIENT, materialAmbient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, materialDiffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, materialSpecular);
	glMaterialf(GL_FRONT, GL_SHININESS, 0.21794872f * 128.0f);	

	if (m_bEnableLight)
	{
		glEnable(GL_LIGHTING);	
		glEnable(GL_LIGHT0);
	}
	
	//÷
	glEnable(GL_POINT_SMOOTH);
	glEnable(GL_LINE_SMOOTH);
	glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 

	// smooth the drawing
	glShadeModel(GL_FLAT);

	// λ
 	glEnable(GL_NORMALIZE);
	glEnable(GL_AUTO_NORMAL);

    // set background to black
	glClearColor(m_BackColorRed, m_BackColorGreen, m_BackColorBlue, m_BackAlpha);
}

void COpenGLView::PrepareAxisLabel()
{
	GLubyte m_RasterFont[][13] = 
	{
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18}, 
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x36, 0x36, 0x36, 0x36}, 
		{0x00, 0x00, 0x00, 0x66, 0x66, 0xff, 0x66, 0x66, 0xff, 0x66, 0x66, 0x00, 0x00}, 
		{0x00, 0x00, 0x18, 0x7e, 0xff, 0x1b, 0x1f, 0x7e, 0xf8, 0xd8, 0xff, 0x7e, 0x18}, 
		{0x00, 0x00, 0x0e, 0x1b, 0xdb, 0x6e, 0x30, 0x18, 0x0c, 0x76, 0xdb, 0xd8, 0x70}, 
		{0x00, 0x00, 0x7f, 0xc6, 0xcf, 0xd8, 0x70, 0x70, 0xd8, 0xcc, 0xcc, 0x6c, 0x38}, 
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x1c, 0x0c, 0x0e}, 
		{0x00, 0x00, 0x0c, 0x18, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x18, 0x0c}, 
		{0x00, 0x00, 0x30, 0x18, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x18, 0x30}, 
		{0x00, 0x00, 0x00, 0x00, 0x99, 0x5a, 0x3c, 0xff, 0x3c, 0x5a, 0x99, 0x00, 0x00}, 
		{0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0xff, 0xff, 0x18, 0x18, 0x18, 0x00, 0x00}, 
		{0x00, 0x00, 0x30, 0x18, 0x1c, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x00, 0x38, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x60, 0x60, 0x30, 0x30, 0x18, 0x18, 0x0c, 0x0c, 0x06, 0x06, 0x03, 0x03}, 
		{0x00, 0x00, 0x3c, 0x66, 0xc3, 0xe3, 0xf3, 0xdb, 0xcf, 0xc7, 0xc3, 0x66, 0x3c}, 
		{0x00, 0x00, 0x7e, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x78, 0x38, 0x18}, 
		{0x00, 0x00, 0xff, 0xc0, 0xc0, 0x60, 0x30, 0x18, 0x0c, 0x06, 0x03, 0xe7, 0x7e}, 
		{0x00, 0x00, 0x7e, 0xe7, 0x03, 0x03, 0x07, 0x7e, 0x07, 0x03, 0x03, 0xe7, 0x7e}, 
		{0x00, 0x00, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0xff, 0xcc, 0x6c, 0x3c, 0x1c, 0x0c}, 
		{0x00, 0x00, 0x7e, 0xe7, 0x03, 0x03, 0x07, 0xfe, 0xc0, 0xc0, 0xc0, 0xc0, 0xff}, 
		{0x00, 0x00, 0x7e, 0xe7, 0xc3, 0xc3, 0xc7, 0xfe, 0xc0, 0xc0, 0xc0, 0xe7, 0x7e}, 
		{0x00, 0x00, 0x30, 0x30, 0x30, 0x30, 0x18, 0x0c, 0x06, 0x03, 0x03, 0x03, 0xff}, 
		{0x00, 0x00, 0x7e, 0xe7, 0xc3, 0xc3, 0xe7, 0x7e, 0xe7, 0xc3, 0xc3, 0xe7, 0x7e}, 
		{0x00, 0x00, 0x7e, 0xe7, 0x03, 0x03, 0x03, 0x7f, 0xe7, 0xc3, 0xc3, 0xe7, 0x7e}, 
		{0x00, 0x00, 0x00, 0x38, 0x38, 0x00, 0x00, 0x38, 0x38, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x30, 0x18, 0x1c, 0x1c, 0x00, 0x00, 0x1c, 0x1c, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x06, 0x0c, 0x18, 0x30, 0x60, 0xc0, 0x60, 0x30, 0x18, 0x0c, 0x06}, 
		{0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x60, 0x30, 0x18, 0x0c, 0x06, 0x03, 0x06, 0x0c, 0x18, 0x30, 0x60}, 
		{0x00, 0x00, 0x18, 0x00, 0x00, 0x18, 0x18, 0x0c, 0x06, 0x03, 0xc3, 0xc3, 0x7e}, 
		{0x00, 0x00, 0x3f, 0x60, 0xcf, 0xdb, 0xd3, 0xdd, 0xc3, 0x7e, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0xc3, 0xc3, 0xc3, 0xc3, 0xff, 0xc3, 0xc3, 0xc3, 0x66, 0x3c, 0x18}, 
		{0x00, 0x00, 0xfe, 0xc7, 0xc3, 0xc3, 0xc7, 0xfe, 0xc7, 0xc3, 0xc3, 0xc7, 0xfe}, 
		{0x00, 0x00, 0x7e, 0xe7, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xe7, 0x7e}, 
		{0x00, 0x00, 0xfc, 0xce, 0xc7, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xc7, 0xce, 0xfc}, 
		{0x00, 0x00, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0xfc, 0xc0, 0xc0, 0xc0, 0xc0, 0xff}, 
		{0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xfc, 0xc0, 0xc0, 0xc0, 0xff}, 
		{0x00, 0x00, 0x7e, 0xe7, 0xc3, 0xc3, 0xcf, 0xc0, 0xc0, 0xc0, 0xc0, 0xe7, 0x7e}, 
		{0x00, 0x00, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xff, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3}, 
		{0x00, 0x00, 0x7e, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x7e}, 
		{0x00, 0x00, 0x7c, 0xee, 0xc6, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06}, 
		{0x00, 0x00, 0xc3, 0xc6, 0xcc, 0xd8, 0xf0, 0xe0, 0xf0, 0xd8, 0xcc, 0xc6, 0xc3}, 
		{0x00, 0x00, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0}, 
		{0x00, 0x00, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xdb, 0xff, 0xff, 0xe7, 0xc3}, 
		{0x00, 0x00, 0xc7, 0xc7, 0xcf, 0xcf, 0xdf, 0xdb, 0xfb, 0xf3, 0xf3, 0xe3, 0xe3}, 
		{0x00, 0x00, 0x7e, 0xe7, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xe7, 0x7e}, 
		{0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xfe, 0xc7, 0xc3, 0xc3, 0xc7, 0xfe}, 
		{0x00, 0x00, 0x3f, 0x6e, 0xdf, 0xdb, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0x66, 0x3c}, 
		{0x00, 0x00, 0xc3, 0xc6, 0xcc, 0xd8, 0xf0, 0xfe, 0xc7, 0xc3, 0xc3, 0xc7, 0xfe}, 
		{0x00, 0x00, 0x7e, 0xe7, 0x03, 0x03, 0x07, 0x7e, 0xe0, 0xc0, 0xc0, 0xe7, 0x7e}, 
		{0x00, 0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0xff}, 
		{0x00, 0x00, 0x7e, 0xe7, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3}, 
		{0x00, 0x00, 0x18, 0x3c, 0x3c, 0x66, 0x66, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3}, 
		{0x00, 0x00, 0xc3, 0xe7, 0xff, 0xff, 0xdb, 0xdb, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3}, 
		{0x00, 0x00, 0xc3, 0x66, 0x66, 0x3c, 0x3c, 0x18, 0x3c, 0x3c, 0x66, 0x66, 0xc3}, 
		{0x00, 0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x3c, 0x3c, 0x66, 0x66, 0xc3}, 
		{0x00, 0x00, 0xff, 0xc0, 0xc0, 0x60, 0x30, 0x7e, 0x0c, 0x06, 0x03, 0x03, 0xff}, 
		{0x00, 0x00, 0x3c, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x3c}, 
		{0x00, 0x03, 0x03, 0x06, 0x06, 0x0c, 0x0c, 0x18, 0x18, 0x30, 0x30, 0x60, 0x60}, 
		{0x00, 0x00, 0x3c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x3c}, 
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc3, 0x66, 0x3c, 0x18}, 
		{0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x38, 0x30, 0x70}, 
		{0x00, 0x00, 0x7f, 0xc3, 0xc3, 0x7f, 0x03, 0xc3, 0x7e, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0xfe, 0xc3, 0xc3, 0xc3, 0xc3, 0xfe, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0}, 
		{0x00, 0x00, 0x7e, 0xc3, 0xc0, 0xc0, 0xc0, 0xc3, 0x7e, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x7f, 0xc3, 0xc3, 0xc3, 0xc3, 0x7f, 0x03, 0x03, 0x03, 0x03, 0x03}, 
		{0x00, 0x00, 0x7f, 0xc0, 0xc0, 0xfe, 0xc3, 0xc3, 0x7e, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x30, 0x30, 0x30, 0x30, 0x30, 0xfc, 0x30, 0x30, 0x30, 0x33, 0x1e}, 
		{0x7e, 0xc3, 0x03, 0x03, 0x7f, 0xc3, 0xc3, 0xc3, 0x7e, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xfe, 0xc0, 0xc0, 0xc0, 0xc0}, 
		{0x00, 0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x00, 0x00, 0x18, 0x00}, 
		{0x38, 0x6c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x00, 0x00, 0x0c, 0x00}, 
		{0x00, 0x00, 0xc6, 0xcc, 0xf8, 0xf0, 0xd8, 0xcc, 0xc6, 0xc0, 0xc0, 0xc0, 0xc0}, 
		{0x00, 0x00, 0x7e, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x78}, 
		{0x00, 0x00, 0xdb, 0xdb, 0xdb, 0xdb, 0xdb, 0xdb, 0xfe, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xfc, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x7c, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x7c, 0x00, 0x00, 0x00, 0x00}, 
		{0xc0, 0xc0, 0xc0, 0xfe, 0xc3, 0xc3, 0xc3, 0xc3, 0xfe, 0x00, 0x00, 0x00, 0x00}, 
		{0x03, 0x03, 0x03, 0x7f, 0xc3, 0xc3, 0xc3, 0xc3, 0x7f, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xe0, 0xfe, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0xfe, 0x03, 0x03, 0x7e, 0xc0, 0xc0, 0x7f, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x1c, 0x36, 0x30, 0x30, 0x30, 0x30, 0xfc, 0x30, 0x30, 0x30, 0x00}, 
		{0x00, 0x00, 0x7e, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x18, 0x3c, 0x3c, 0x66, 0x66, 0xc3, 0xc3, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0xc3, 0xe7, 0xff, 0xdb, 0xc3, 0xc3, 0xc3, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0xc3, 0x66, 0x3c, 0x18, 0x3c, 0x66, 0xc3, 0x00, 0x00, 0x00, 0x00}, 
		{0xc0, 0x60, 0x60, 0x30, 0x18, 0x3c, 0x66, 0x66, 0xc3, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0xff, 0x60, 0x30, 0x18, 0x0c, 0x06, 0xff, 0x00, 0x00, 0x00, 0x00}, 
		{0x00, 0x00, 0x0f, 0x18, 0x18, 0x18, 0x38, 0xf0, 0x38, 0x18, 0x18, 0x18, 0x0f}, 
		{0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18}, 
		{0x00, 0x00, 0xf0, 0x18, 0x18, 0x18, 0x1c, 0x0f, 0x1c, 0x18, 0x18, 0x18, 0xf0}, 
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x8f, 0xf1, 0x60, 0x00, 0x00, 0x00} 
	};

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	m_nFontOffset = glGenLists(128);
	for (GLuint i = 32; i < 127; i++)
	{
		glNewList(i + m_nFontOffset, GL_COMPILE);
		glBitmap(8, 13, 0.0f, 0.0f, 0.0f, 0.0f, m_RasterFont[i-32]);
		glEndList();
	}
}


void COpenGLView::PrintAxisLabel(const char* str)
{
	glPushAttrib(GL_LIST_BIT);
	glListBase(m_nFontOffset);
	glCallLists(static_cast<GLsizei>(strlen(str)), GL_UNSIGNED_BYTE, (GLubyte*)str);
	glPopAttrib();
}


void COpenGLView::BuildAxisesList()
{
	m_nAxisesList = glGenLists(1);
	glNewList(m_nAxisesList, GL_COMPILE);
	glPushAttrib(GL_CURRENT_BIT | GL_LINE_BIT);

	glLineWidth(2.0f);
	glBegin(GL_LINES);
	glColor3ub(255, 0, 0); 
	glVertex3f(0.0, 0.0, 0.0);
	glVertex3f(0.1, 0.0, 0.0);

	glColor3ub(0, 255, 0);
	glVertex3f(0.0, 0.0, 0.0);
	glVertex3f(0.0, 0.1, 0.0);

	glColor3ub(0, 0, 255);   
	glVertex3f(0.0, 0.0, 0.0);
	glVertex3f(0.0, 0.0, 0.1);
	glEnd();

	//ϵļͷע"X,Y,Z"
	glColor3ub(255, 255, 0);	
	glRasterPos3f(0.15f, 0.00f, 0.00f);	PrintAxisLabel("X");
	glRasterPos3f(0.00f, 0.15f, 0.00f);	PrintAxisLabel("Y");	
	glRasterPos3f(0.00f, 0.00f, 0.15f);	PrintAxisLabel("Z");
	//x
	glPushMatrix();
	glColor3ub(255, 0, 0);
	glTranslatef(0.1, 0.0, 0.0);
	glRotatef(90.0, 0.0, 1.0, 0.0);
	glutSolidCone(0.01, 0.04, 20, 20);
	glPopMatrix();
	//y
	glPushMatrix();
	glColor3ub(0, 255, 0);
	glTranslatef(0.0, 0.1, 0.0);
	glRotatef(-90.0, 1.0, 0.0, 0.0);
	glutSolidCone(0.01, 0.04, 20, 20);
	glPopMatrix();
	//z
	glPushMatrix();
	glColor3ub(0, 0, 255); 
	glTranslatef(0.0, 0.0, 0.1);
	glutSolidCone(0.01, 0.04, 20, 20);	
	glPopMatrix();

	glColor3ub(255, 255, 255);
	glutSolidSphere(0.008f, 20, 20);

	glPopAttrib();
	glEndList();
}

/////////////////////////////////////////////////
// default initializing of the drawing environment
void COpenGLView::SetUpDrawingEnvironment()
{
	// This function must be changed if you want
	// to bargain for other specs than those 
	// specified as initial values to
	// PIXELFORMATDESCRIPTOR below
	
	// make sure we do only one initialization
	static BOOL m_bIsInitialized = FALSE;
	ASSERT(!m_bIsInitialized);
	m_bIsInitialized = TRUE;	
	
	static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
			1,                              // version number
			PFD_DRAW_TO_WINDOW |            // support window
			PFD_SUPPORT_OPENGL |          // support OpenGL
			PFD_DOUBLEBUFFER,             // double buffered
			PFD_TYPE_RGBA,                  // RGBA type
			24,                             // 24-bit color depth
			0, 0, 0, 0, 0, 0,               // color bits ignored
			0,                              // no alpha buffer
			0,                              // shift bit ignored
			0,                              // no accumulation buffer
			0, 0, 0, 0,                     // accum bits ignored
			32,                             // 32-bit z-buffer
			0,                              // no stencil buffer
			0,                              // no auxiliary buffer
			PFD_MAIN_PLANE,                 // main layer
			0,                              // reserved
			0, 0, 0                         // layer masks ignored
    };
	
	int nPixelFormat;
	
	nPixelFormat = ChoosePixelFormat(m_hDC, &pfd);
	SetPixelFormat(m_hDC, nPixelFormat, &pfd);
	
	DescribePixelFormat(m_hDC,nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
	if(pfd.dwFlags & PFD_NEED_PALETTE)
		AfxMessageBox(_T("Only 256 colors available."));
	
	// create rendering context associated with  windows dc
	m_hRC = wglCreateContext(m_hDC);
	// keep it current in the windows lifetime
	// if you do not change it in subclass
	wglMakeCurrent(m_hDC, m_hRC);
}


void COpenGLView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	m_ptLeftPrevDownPos = point;
	m_bLeftButtonDown = true;
	
	SetCapture();	
	
	CView::OnLButtonDown(nFlags, point);
}


void COpenGLView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	m_bLeftButtonDown = false;
	m_bRightButtonDown = false;
	
	ReleaseCapture();
	
	CView::OnLButtonUp(nFlags, point);
}



void COpenGLView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	m_ptRightPrevDownPos = point;
	m_bRightButtonDown = true;

	SetCapture();
	
	CView::OnRButtonDown(nFlags, point);
}

void COpenGLView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	m_bLeftButtonDown = false;
	m_bRightButtonDown = false;
	
	ReleaseCapture();
	
	CView::OnRButtonUp(nFlags, point);
}


void COpenGLView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// Both : x / y translation
	if(m_bLeftButtonDown && m_bRightButtonDown)
	{
		m_xTranslation -= (float)(m_ptLeftPrevDownPos.x - point.x) * m_SpeedTranslation;
		m_yTranslation += (float)(m_ptLeftPrevDownPos.y - point.y) * m_SpeedTranslation;
		
		m_ptLeftPrevDownPos = point;		
		RedrawWindow();
	}
	
	else if(m_bLeftButtonDown)// Left : rotation
	{
		m_yRotation -= (float)(m_ptLeftPrevDownPos.x - point.x) * m_SpeedRotation;
		m_xRotation -= (float)(m_ptLeftPrevDownPos.y - point.y) * m_SpeedRotation;

		m_ptLeftPrevDownPos = point;		
		RedrawWindow();
	}
	
	else if(m_bRightButtonDown)// Right : z translation
	{
		if (abs(m_ptRightPrevDownPos.y - point.y) > 2)
		{
			if (m_ptRightPrevDownPos.y - point.y < 0) 
				m_Scaling /= m_SpeedScaling;
			else
				m_Scaling *= m_SpeedScaling;
		}

		m_ptRightPrevDownPos = point;		
		RedrawWindow();
	}
	
	CView::OnMouseMove(nFlags, point);
}



void COpenGLView::EnableLighting(bool b)
{
	m_bEnableLight = b;
	if (m_bEnableLight)
		glEnable(GL_LIGHTING);
	else
		glDisable(GL_LIGHTING);

	RedrawWindow();
}


void COpenGLView::EnableShowAxises(bool b)
{
	m_bShowAxises = b;
	RedrawWindow();
}
void COpenGLView::ViewFitAll() 
{
	m_Scaling = 1.0f;
	m_xTranslation = 0.0f;
	m_yTranslation = 0.0f;
	m_zTranslation = 0.0f;		

	RedrawWindow();	
}

void COpenGLView::ViewPosTop() 
{
	m_xRotation = 0.0;
	m_yRotation = 0.0;
	m_zRotation = 0.0;

	InvalidateRect(NULL);
}


void COpenGLView::ViewPosBottom() 
{
	m_xRotation = -180.0;
	m_yRotation = 0.0;
	m_zRotation = 0.0;

	InvalidateRect(NULL);
}


void COpenGLView::ViewPosFront() 
{
	m_xRotation = -90.0;
	m_yRotation = 0.0;
	m_zRotation = 0.0;

	InvalidateRect(NULL);
}

void COpenGLView::ViewPosBack() 
{
	m_xRotation = -90.0;
	m_yRotation = 0.0;
	m_zRotation = -180.0;

	InvalidateRect(NULL);
}

void COpenGLView::ViewPosLeft() 
{
	m_xRotation = -90.0;
	m_yRotation = 0.0;
	m_zRotation = 90.0;

	InvalidateRect(NULL);
}

void COpenGLView::ViewPosRight() 
{
	m_xRotation = -90.0;
	m_yRotation = 0.0;
	m_zRotation = -90.0;

	InvalidateRect(NULL);
}


void COpenGLView::ViewPosTrimetric()
{
	m_xRotation = -45.0;
	m_yRotation = 0.0;
	m_zRotation = -45.0;

	InvalidateRect(NULL);
}

