// OperationView.cpp : implementation file
//

#include "stdafx.h"
#include "Segmentation.h"
#include "SegmentationView.h"
#include "SegmentationDoc.h"
#include "TriMesh.h"
#include "MainFrm.h"
#include "OperationView.h"


// COperationView

IMPLEMENT_DYNCREATE(COperationView, CFormView)

COperationView::COperationView()
	: CFormView(COperationView::IDD)
	, m_featureCriteria(FC_DIHEDRAL)
	, m_featureEdgeRatio(5)
	, m_featureEdgeDihedral(30)
	, m_pDrawView(NULL)
	, m_bShowSTLModel(true)
	, m_bShowFeatureEdges (false)
	, m_bShowSegmentation (true)
	, m_bShowContour (true)
	, m_bShowInnerSkeleton (true)
	, m_bShowContourSkeleton (true)
	, m_bShowAllOffset(true)
	, m_bShowPolygonDelaunayGraph(true)
	, m_bShowNarrowAreas(true)
	, m_dOffset(0)
	, m_dRegionBBoxWidth(0)
	, m_dRegionBBoxHeight(0)
	, m_dNarrowAreaWidthThreshold(0)
	, m_nWhichOffsetContourToShow(0)
	, m_bInitiallized(false)
{
}

COperationView::~COperationView()
{
}

void COperationView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDIT_FEATURE_EDGE_RATIO, m_featureEdgeRatio);
	DDX_Text(pDX, IDC_EDIT_FEATURE_EDGE_DIHEDRAL, m_featureEdgeDihedral);

	DDX_Text(pDX, IDC_EDIT_REGION_BBOX_WIDTH, m_dRegionBBoxWidth);
	DDX_Text(pDX, IDC_EDIT_REGION_BBOX_HEIGHT, m_dRegionBBoxHeight);

	DDX_Text(pDX, IDC_EDIT_OFFSET, m_dOffset);
	DDX_Text(pDX, IDC_EDIT_NARROW_AREA_WIDTH, m_dNarrowAreaWidthThreshold);

	DDX_Text(pDX, IDC_EDIT_WHICH_OFFSET_TO_SHOW, m_nWhichOffsetContourToShow);
	DDV_MinMaxInt(pDX, m_nWhichOffsetContourToShow, 0, 5000);
}

BEGIN_MESSAGE_MAP(COperationView, CFormView)
	ON_BN_CLICKED(IDC_BUTTON_SEGMENTATION_APPLY, &COperationView::OnBnClickedButtonSegmentationApply)
	ON_BN_CLICKED(IDC_RADIO_FEATURE_EDGE_RATIO, &COperationView::OnBnClickedRadioFeatureEdgeRatio)
	ON_BN_CLICKED(IDC_RADIO_FEATURE_EDGE_DIHEDRAL, &COperationView::OnBnClickedRadioFeatureEdgeDihedral)
	ON_BN_CLICKED(IDC_CHECK_SHOW_FEATURE_EDGES, &COperationView::OnBnClickedCheckShowFeatureEdges)
	ON_BN_CLICKED(IDC_CHECK_SHOW_CLORED_RESULT, &COperationView::OnBnClickedCheckShowCloredResult)
	ON_BN_CLICKED(IDC_BUTTON_CREATE_INNER_SKELETON, &COperationView::OnBnClickedButtonCreateInnerSkeleton)
	ON_BN_CLICKED(IDC_BUTTON_CREATE_OFFSET, &COperationView::OnBnClickedButtonCreateOffset)
	ON_BN_CLICKED(IDC_CHECK_SHOW_CONTOUR, &COperationView::OnBnClickedCheckShowContour)
	ON_BN_CLICKED(IDC_CHECK_SHOW_INNER_SKELETON, &COperationView::OnBnClickedCheckShowInnerSkeleton)
	ON_BN_CLICKED(IDC_RADIO_SHOW_ALL_OFFSET, &COperationView::OnBnClickedRadioShowAllOffset)
	ON_BN_CLICKED(IDC_RADIO_SHOW_SPECIFIED_OFFSET, &COperationView::OnBnClickedRadioShowSpecifiedOffset)
	ON_BN_CLICKED(IDC_CHECK_SHOW_CONTOUR_SKELETON, &COperationView::OnBnClickedCheckShowContourSkeleton)
	ON_BN_CLICKED(IDC_CHECK_SHOW_DELAUNAY_GRAPH, &COperationView::OnBnClickedCheckShowDelaunayGraph)
	ON_BN_CLICKED(IDC_CHECK_SHOW_NARROW_AREA, &COperationView::OnBnClickedCheckShowNarrowAreas)
	ON_BN_CLICKED(IDC_BUTTON_CREATE_POLYGON_DELAUNAY_GRAPH, &COperationView::OnBnClickedButtonCreatePolygonDelaunayGraph)
	ON_BN_CLICKED(IDC_BUTTON_DETECT_NARROW_AREA, &COperationView::OnBnClickedButtonDetectNarrowArea)
	ON_EN_CHANGE(IDC_EDIT_WHICH_OFFSET_TO_SHOW, &COperationView::OnEnChangeEditWhichOffsetToShow)
END_MESSAGE_MAP()


// COperationView diagnostics

#ifdef _DEBUG
void COperationView::AssertValid() const
{
	CFormView::AssertValid();
}

#ifndef _WIN32_WCE
void COperationView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}
#endif
#endif //_DEBUG


// COperationView message handlers

void COperationView::OnBnClickedButtonSegmentationApply()
{
	UpdateData(TRUE);
	if (m_featureEdgeRatio <= 0 || m_featureEdgeRatio >= 100) {
		AfxMessageBox(_T("ЧĲ"));
		return;
	}

	TriMesh* pMesh = m_pDoc->m_pTriMesh;
	if (pMesh == NULL) {
		AfxMessageBox(_T("ģͲڣ"));
		return;
	}

	m_featureEdgeRatio /= 100.0;

	m_pDoc->m_segmentation.SetTarget(pMesh);
	switch (m_featureCriteria)
	{
	case FC_RATIO:
		m_pDoc->m_segmentation.Apply(FC_RATIO, m_featureEdgeRatio);
		break;
	case FC_DIHEDRAL:
		m_pDoc->m_segmentation.Apply(FC_DIHEDRAL, m_featureEdgeDihedral);
		break;
	default:
		break;
	}

	m_pDoc->m_pTriMesh->SetShowModel(false);
	m_pDoc->m_segmentation.SetShowSegmentation(true);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_CLORED_RESULT))->SetCheck(1);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();

	// TODO: Add your specialized code here and/or call the base class
	m_pDrawView = ((CMainFrame*)AfxGetMainWnd())->m_pDrawView;
	m_pDoc = m_pDrawView->GetDocument();

	((CButton*)GetDlgItem(IDC_RADIO_FEATURE_EDGE_DIHEDRAL))->SetCheck(1);
	((CButton*)GetDlgItem(IDC_EDIT_FEATURE_EDGE_RATIO))->EnableWindow(FALSE);

	((CButton*)GetDlgItem(IDC_CHECK_SHOW_FEATURE_EDGES))->SetCheck(m_bShowFeatureEdges ? 1 : 0);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_CLORED_RESULT))->SetCheck(m_bShowSegmentation ? 1 : 0);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_CONTOUR))->SetCheck(m_bShowContour ? 1 : 0);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_INNER_SKELETON))->SetCheck(m_bShowInnerSkeleton ? 1 : 0);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_CONTOUR_SKELETON))->SetCheck(m_bShowContourSkeleton ? 1 : 0);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_DELAUNAY_GRAPH))->SetCheck(m_bShowPolygonDelaunayGraph ? 1 : 0);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_NARROW_AREA))->SetCheck(m_bShowNarrowAreas ? 1 : 0);

	((CButton*)GetDlgItem(IDC_RADIO_SHOW_ALL_OFFSET))->SetCheck(m_bShowAllOffset ? 1 : 0);
	((CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_WHICH_OFFSET_TO_SHOW))->SetRange(0, 5000);	

	m_bInitiallized = true;
}

void COperationView::OnBnClickedRadioFeatureEdgeRatio()
{
	m_featureCriteria = FC_RATIO;
	((CButton*)GetDlgItem(IDC_EDIT_FEATURE_EDGE_RATIO))->EnableWindow(TRUE);
	((CButton*)GetDlgItem(IDC_EDIT_FEATURE_EDGE_DIHEDRAL))->EnableWindow(FALSE);
}

void COperationView::OnBnClickedRadioFeatureEdgeDihedral()
{
	m_featureCriteria = FC_DIHEDRAL;
	((CButton*)GetDlgItem(IDC_EDIT_FEATURE_EDGE_RATIO))->EnableWindow(FALSE);
	((CButton*)GetDlgItem(IDC_EDIT_FEATURE_EDGE_DIHEDRAL))->EnableWindow(TRUE);
}


void COperationView::OnBnClickedCheckShowFeatureEdges()
{
	m_bShowFeatureEdges = !m_bShowFeatureEdges;
	if (m_pDoc->m_pTriMesh)
		m_pDoc->m_pTriMesh->SetShowFeatureEdges(m_bShowFeatureEdges);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_FEATURE_EDGES))->SetCheck(m_bShowFeatureEdges ? 1 : 0);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::OnBnClickedCheckShowCloredResult()
{
	m_bShowSegmentation = !m_bShowSegmentation;
	m_bShowSTLModel = !m_bShowSegmentation;	
	m_pDoc->m_segmentation.SetShowSegmentation(m_bShowSegmentation);
	if (m_pDoc->m_pTriMesh)
		m_pDoc->m_pTriMesh->SetShowModel(m_bShowSTLModel);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_CLORED_RESULT))->SetCheck(m_bShowSegmentation ? 1 : 0);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::SetRegionDimension(double width, double height)
{
	m_dRegionBBoxWidth  = width;
	m_dRegionBBoxHeight = height;

	UpdateData(FALSE);
}


void COperationView::OnBnClickedCheckShowContour()
{
	m_bShowContour = !m_bShowContour;
	m_pDoc->m_polygonSet.SetShowPolygons(m_bShowContour);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_CONTOUR))->SetCheck(m_bShowContour ? 1 : 0);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}


void COperationView::OnBnClickedCheckShowInnerSkeleton()
{
	m_bShowInnerSkeleton = !m_bShowInnerSkeleton;
	m_pDoc->m_straightSkeleton.SetShowInnerSkeleton(m_bShowInnerSkeleton);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_INNER_SKELETON))->SetCheck(m_bShowInnerSkeleton ? 1 : 0);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::OnBnClickedCheckShowContourSkeleton()
{
	m_bShowContourSkeleton = !m_bShowContourSkeleton;
	m_pDoc->m_straightSkeleton.SetShowContourSkeleton(m_bShowContourSkeleton);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_CONTOUR_SKELETON))->SetCheck(m_bShowContourSkeleton ? 1 : 0);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::OnBnClickedRadioShowAllOffset()
{
	m_bShowAllOffset = true;
	m_pDoc->m_straightSkeleton.SetShowAllOffset(m_bShowAllOffset);
	((CButton*)GetDlgItem(IDC_RADIO_SHOW_ALL_OFFSET))->SetCheck(m_bShowAllOffset ? 1 : 0);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::OnEnChangeEditWhichOffsetToShow()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
	if (m_bInitiallized)
	{
		UpdateData(TRUE);
		m_pDoc->m_straightSkeleton.SetShowSpecifiedOffset(m_nWhichOffsetContourToShow);
		m_pDrawView->InvalidateRect(NULL, FALSE);
	}
}


void COperationView::OnBnClickedRadioShowSpecifiedOffset()
{
	m_bShowAllOffset = false;
	m_pDoc->m_straightSkeleton.SetShowAllOffset(m_bShowAllOffset);
	m_pDoc->m_straightSkeleton.SetShowSpecifiedOffset(m_nWhichOffsetContourToShow);
	((CButton*)GetDlgItem(IDC_RADIO_SHOW_SPECIFIED_OFFSET))->SetCheck(m_bShowAllOffset ? 0 : 1);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::OnBnClickedCheckShowDelaunayGraph()
{
	m_bShowPolygonDelaunayGraph = !m_bShowPolygonDelaunayGraph;
	m_pDoc->m_polygonDelaunayGraph.SetShowGraph(m_bShowPolygonDelaunayGraph);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_DELAUNAY_GRAPH))->SetCheck(m_bShowPolygonDelaunayGraph ? 1 : 0);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::OnBnClickedCheckShowNarrowAreas()
{
	m_bShowNarrowAreas = !m_bShowNarrowAreas;
	m_pDoc->m_polygonDelaunayGraph.SetShowNarrowAreas(m_bShowNarrowAreas);
	((CButton*)GetDlgItem(IDC_CHECK_SHOW_NARROW_AREA))->SetCheck(m_bShowNarrowAreas ? 1 : 0);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}


void COperationView::OnBnClickedButtonCreateInnerSkeleton()
{
	bool dataExist = (m_pDoc->m_polygonSet.m_polygons.size() > 0);
	if (!dataExist)	{
		AfxMessageBox(_T("ݲڣ"));
		return;
	}

	StraightSkeleton&  sskel = m_pDoc->m_straightSkeleton;
	sskel.SetPolygonData(m_pDoc->m_polygonSet.m_polygons);
	sskel.CreateSkeleton();
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::OnBnClickedButtonCreateOffset()
{
	UpdateData(TRUE);

	bool dataExist = (m_pDoc->m_polygonSet.m_polygons.size() > 0);
	if (!dataExist)	{
		AfxMessageBox(_T("ݲڣ"));
		return;
	}

	if (m_dOffset <=0 || m_dOffset >= min(m_dRegionBBoxHeight, m_dRegionBBoxWidth) ) {
		AfxMessageBox(_T("ЧĲ"));
		return;
	}

	StraightSkeleton&  sskel = m_pDoc->m_straightSkeleton;
	if (sskel.m_SSkels.empty()) {
		AfxMessageBox(_T("ȼǼܣ"));
		return;
	}

	sskel.CreateOffset(m_dOffset);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::OnBnClickedButtonCreatePolygonDelaunayGraph()
{
	bool dataExist = (m_pDoc->m_polygonSet.m_polygons.size() > 0);
	if (!dataExist)	{
		AfxMessageBox(_T("ݲڣ"));
		return;
	}

	PolygonDelaunayGraph& pdg = m_pDoc->m_polygonDelaunayGraph;
	pdg.SetPolygonData(m_pDoc->m_polygonSet.m_polygons);
	pdg.Create();
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

void COperationView::OnBnClickedButtonDetectNarrowArea()
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);

	if (m_dNarrowAreaWidthThreshold <= 0 || m_dNarrowAreaWidthThreshold >= min(m_dRegionBBoxHeight, m_dRegionBBoxWidth)) {
		AfxMessageBox(_T("ЧĲ"));
		return;
	}

	PolygonDelaunayGraph& pdg = m_pDoc->m_polygonDelaunayGraph;
	if (!pdg.m_sdg.is_valid(true, 1)) {
		AfxMessageBox(_T("ȼDelaunay graph!"));
		return;
	}

	pdg.DetectNarrowAreas(m_dNarrowAreaWidthThreshold);
	m_pDrawView->InvalidateRect(NULL, FALSE);
}

