/*
* Copyright (c) 2007, Shenyang Institute of Automation, Chinese Academy
* of Sciences. , all rights reserved.
* 
* Filename     : Point2D.cpp
* Project	   : xMesh library
* Abstract     : Implementation of the Point2D class
* Version      : 1.0
* Author			: , nanlliang@sia.cn
* Original Author   : , nanlliang@sia.cn
* Accomplished date : Dec. 29, 2007
*/

//-----------------------------------------------------------------


#include "StdAfx.h"
#include "Point2D.h"
#include "Vector2D.h"


Point2D::Point2D() : x(0), y(0)
{}

Point2D::Point2D(double _x, double _y) : x(_x), y(_y)
{}

Point2D::Point2D(double* p)
{
	x = p[0]; y = p[1];
}
Point2D::Point2D(const Point2D& p)
{
	x = p.GetX(); y = p.GetY();
}

Point2D::Point2D(const Vector2D& v)
{
	x = v.GetX(); y = v.GetY();
}

void Point2D::SetValue(double _x, double _y)
{
	x = _x; y = _y;
}

void Point2D::SetValue(const Point2D& p) 
{ 
	x = p.GetX(); y = p.GetY();
}

void Point2D::SetValue(const Vector2D& v)
{ 
	x = v.GetX(); y = v.GetY();
}

void Point2D::SetZero()
{
	SetValue(0, 0); 
}

void  Point2D::SetX(double _x) { x = _x; }

void  Point2D::SetY(double _y) { y = _y; }

double Point2D::GetX() const { return x; }

double Point2D::GetY() const { return y; }

double Point2D::DistanceTo(const Point2D& p) const
{
	return sqrt( (x - p.x) * (x - p.x) +
		         (y - p.y) * (y - p.y) );
}

Point2D Point2D::Origin()
{
	return Point2D(0, 0);
}

Vector2D Point2D::Vector() const
{
	return Vector2D(x, y);
}

void Point2D::operator=(const Point2D& p)
{ 
	x = p.GetX(); y = p.GetY();
}

Point2D Point2D::operator+(const Vector2D& v) const
{
	return Point2D(x + v.GetX(), y + v.GetY());
}

Point2D Point2D::operator-(const Vector2D& v) const 
{
	return Point2D(x - v.GetX(), y - v.GetY());
}

Point2D Point2D::operator*(const double s) const
{
	return Point2D(x * s, y * s);
}

Point2D Point2D::operator/(const double s) const
{
	return Point2D(x / s, y / s);
}

void Point2D::operator+=(const Vector2D& v)
{
	x += v.GetX(); y += v.GetY();
}

void Point2D::operator-=(const Vector2D& v)
{
	x -= v.GetX(); y -= v.GetY();
}

void Point2D::operator*=(const double s)
{
	(*this) = (*this) * s;
}	

void Point2D::operator/=(const double s)
{
	(*this) = (*this) / s;
}

double Point2D::operator [] (const int i) const
{
	switch (i)
	{
	case 0:	return x;
	case 1:	return y;
	default: break;
	}
	return x;
}

double& Point2D::operator [] (const int i) 
{
	switch (i)
	{
	case 0:	return x;
	case 1:	return y;
	default: break;
	}
	return x;
}

bool Point2D::operator <(const Point2D &p) const
{
	return ( ( x < p.x ) ||
		((x == p.x) && (y < p.y)));
}

//ǳΪغ
bool Point2D::operator ==(const Point2D &p) const
{
	return this->DistanceTo(p) == 0 ;
}
