
/*
* Copyright (c) 2007, Shenyang Institute of Automation, Chinese Academy
* of Sciences. , all rights reserved.
* 
* Filename     : Point3D.cpp
* Project	   : xMesh library
* Abstract     : Implementation of the Point3D class
* Version      : 1.0
* Author			: , nanlliang@sia.cn
* Original Author   : , nanlliang@sia.cn
* Accomplished date : Dec. 29, 2007
*/

//-----------------------------------------------------------------

#include "StdAfx.h"
#include "Point3D.h"
#include "Vector3D.h"


using namespace std;



Point3D::Point3D() : x(0), y(0), z(0)
{}

Point3D::Point3D(double _x, double _y, double _z) : x(_x), y(_y), z(_z) 
{}

Point3D::Point3D(double *p)
{
	x = p[0]; y = p[1]; z = p[2];
}

Point3D::Point3D(const Point3D& p)
{
	x = p.GetX(); y = p.GetY(); z = p.GetZ(); 
}

Point3D::Point3D(const Vector3D& v)
{
	x = v.GetX(); y = v.GetY(); z = v.GetZ(); 
}

void Point3D::SetValue(double _x, double _y, double _z)
{
	x = _x; y = _y; z = _z;
}

void Point3D::SetValue(const Point3D& p) 
{ 
	x = p.GetX(); y = p.GetY(); z = p.GetZ(); 
}

void Point3D::SetValue(const Vector3D& v)
{ 
	x = v.GetX(); y = v.GetY(); z = v.GetZ(); 
}

void Point3D::SetZero()
{
	SetValue(0, 0, 0); 
}

void  Point3D::SetX(double _x) { x = _x; }

void  Point3D::SetY(double _y) { y = _y; }

void  Point3D::SetZ(double _z) { z = _z; }

double Point3D::GetX() const { return x; }

double Point3D::GetY() const { return y; }

double Point3D::GetZ() const { return z; }

double Point3D::DistanceTo(const Point3D& p) const
{
	return sqrt( (x - p.x) * (x - p.x) +
		         (y - p.y) * (y - p.y) + 
		         (z - p.z) * (z - p.z) );
}

Point3D Point3D::Origin()
{
	return Point3D(0, 0, 0);
}

Vector3D Point3D::Vector() const
{
	return Vector3D(x, y, z);
}

void Point3D::operator=(const Point3D& p)
{ 
	x = p.GetX(); y = p.GetY(); z = p.GetZ(); 
}

Point3D Point3D::operator+(const Vector3D& v) const
{
	return Point3D(x + v.GetX(), y + v.GetY(), z + v.GetZ());
}

Point3D Point3D::operator-(const Vector3D& v) const 
{
	return Point3D(x - v.GetX(), y - v.GetY(), z - v.GetZ());
}

Point3D Point3D::operator*(const double s) const
{
	return Point3D(x * s, y * s, z * s);
}

Point3D Point3D::operator/(const double s) const
{
	return Point3D(x / s, y / s, z / s);
}

void Point3D::operator+=(const Vector3D& v)
{
	x += v.GetX(); y += v.GetY(); z += v.GetZ(); 
}

void Point3D::operator-=(const Vector3D& v)
{
	x -= v.GetX(); y -= v.GetY(); z -= v.GetZ(); 
}

void Point3D::operator*=(const double s)
{
	(*this) = (*this) * s;
}	

void Point3D::operator/=(const double s)
{
	(*this) = (*this) / s;
}

double Point3D::operator [] (const int i) const
{
	switch (i)
	{
	case 0:	return x;
	case 1:	return y;
	case 2:	return z;
	default: break;
	}
	return x;
}

double& Point3D::operator [] (const int i) 
{
	switch (i)
	{
	case 0:	return x;
	case 1:	return y;
	case 2:	return z;
	default: break;
	}
	return x;
}

bool Point3D::operator <(const Point3D &p) const
{
	return ( ( x < p.x ) ||
		    ((x == p.x) && (y < p.y)) ||
		    ((x == p.x) && (y == p.y) && (z < p.z)));
}

//ǳΪغ
bool Point3D::operator ==(const Point3D &p) const
{
	return this->DistanceTo(p) == 1.0e-6;
}
