/*
* Copyright (c) 2007, Shenyang Institute of Automation, Chinese Academy
* of Sciences. , all rights reserved.
* 
* Filename     : Point3D.h
* Project	   : xMesh library
* Abstract     : Interface for the Point3D class
* Version      : 1.0
* Author			: , nanlliang@sia.cn
* Original Author   : , nanlliang@sia.cn
* Accomplished date : Dec. 29, 2007
*/

//-----------------------------------------------------------------

#ifndef _POINT3D_H_
#define _POINT3D_H_


class Vector3D;

class Point3D
{
public:
	Point3D();
	Point3D(double _x, double _y, double _z);
	Point3D(double *p);
	Point3D(const Point3D& p);
	Point3D(const Vector3D& v);
	~Point3D() {};	

	void   SetValue(double _x, double _y, double _z);
	void   SetValue(const Point3D& p);
	void   SetValue(const Vector3D& v);
	void   SetZero();

	void   SetX(double _x);
	void   SetY(double _y);
	void   SetZ(double _z);

	double GetX() const;
	double GetY() const;
	double GetZ() const;

	double DistanceTo(const Point3D& p) const;
	//double DistanceTo(const Plane& plane) const;

	Point3D  Origin();
	Vector3D Vector() const;

	void operator=(const Point3D& p);
	
	Point3D operator+(const Vector3D& v) const;
	Point3D operator-(const Vector3D& v) const;
	Point3D operator*(const double s) const;
	Point3D operator/(const double s) const;

	void operator+=(const Vector3D& v);
	void operator-=(const Vector3D& v);
	void operator*=(const double s);
	void operator/=(const double s);

	double  operator [] (const int i) const;
	double& operator [] (const int i);

	bool operator <(const Point3D &p) const;
	bool operator ==(const Point3D &p) const;

public:
	double x;
	double y;
	double z;
};

#endif

