#pragma once

#include "CGAL_Object.h"
#include "Segment_Delaunay_graph_vertex_base_with_info_2.h"

#include <CGAL/Segment_Delaunay_graph_2.h>
#include <CGAL/Segment_Delaunay_graph_hierarchy_2.h>
#include <CGAL/Segment_Delaunay_graph_filtered_traits_2.h>

#include <CGAL/CORE_Expr.h>
typedef CGAL::Simple_cartesian<CORE::Expr>  EK;

typedef CGAL::Field_with_sqrt_tag  MTag;
typedef CGAL::Field_with_sqrt_tag  EMTag;
typedef CGAL::Tag_false			   ITag;
typedef CGAL::Tag_true			   STag;

typedef CGAL::Segment_Delaunay_graph_filtered_traits_without_intersections_2<K, MTag, EK, EMTag>     Gt;

typedef Gt::Point_2            Point_2;
typedef Gt::Segment_2          Segment_2;
typedef Gt::Site_2             Site;

typedef CGAL::Segment_Delaunay_graph_storage_traits_2<Gt>             ST;
typedef CGAL::Segment_Delaunay_graph_vertex_base_2<ST>                Vb;
typedef CGAL::Segment_Delaunay_graph_vertex_base_with_info_2<Vb,int>  Vbi;
typedef CGAL::Segment_Delaunay_graph_hierarchy_vertex_base_2<Vbi>     Vbh;
typedef CGAL::Triangulation_face_base_2<Gt>                           Fb;
typedef CGAL::Triangulation_data_structure_2<Vbh,Fb>                  DS;

typedef CGAL::Segment_Delaunay_graph_hierarchy_2<Gt,ST,STag,DS>       SDG_2;
//typedef CGAL::Segment_Delaunay_graph_2<Gt,ST,DS>					  SDG_2;


//////////////////////////////////////////////////////////////////////////
#include "StraightSkeleton.h"

class PolygonDelaunayGraph : public CGAL_Object
{
public:
	PolygonDelaunayGraph(void);
	~PolygonDelaunayGraph(void) {}

	virtual void Draw();
	virtual void Clear();

	void SetShowGraph(bool b);
	void SetShowNarrowAreas(bool b);
	void SetPolygonData(const std::list<std::list<Point_2>>& polygons);
	void Create();

	void DetectNarrowAreas(double widthThreshold);

private:
	void DrawEdge(const SDG_2::Finite_edges_iterator& eit);

	void FindAllPointsAndTheirSites();
	void RecordPoint(Point_2 p);

	virtual void BuildDisplayListObject();
	void		 BuildDisplayListNarrowAreas();

public:
	std::vector<Polygon_2>				m_polygons;
	SDG_2								m_sdg;

	bool								m_bShowGraph;
	bool								m_bShowNarrowAreas;

	GLuint								m_nDisplayListNarrowAreas;
	double								m_dNarrowThreshold;


	typedef std::pair<Point_2, SDG_2::Site_2>   PointAndSite;
	std::vector<PointAndSite>					m_narrowAreaPointsAndSites;
	std::map<Point_2, SDG_2::Site_2>			m_allPointsAndNearestSites;
};
