#include "StdAfx.h"
#include "PolygonSet.h"

PolygonSet::PolygonSet(void)
: m_bShowPolygons(true)
{
}

PolygonSet::~PolygonSet(void)
{
}



void PolygonSet::Clear()
{
	m_polygons.clear();

	glDeleteLists(m_nDisplayListObject, 1);
	m_nDisplayListObject = 0;
}


void PolygonSet::SetShowPolygons(bool b)
{
	m_bShowPolygons = b;
}


void PolygonSet::Draw() 
{
	if (m_polygons.size() > 0  && m_bShowPolygons)
		glCallList(m_nDisplayListObject);
}


void PolygonSet::BuildDisplayListObject()
{
	m_nDisplayListObject = glGenLists(1);
	glNewList(m_nDisplayListObject, GL_COMPILE);
	::glPushMatrix();
	::glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_POINT_BIT | GL_LINE_BIT | GL_LIGHTING_BIT);

	::glScalef(m_fFitScale, m_fFitScale, m_fFitScale);
	::glTranslated(-m_center.x(), -m_center.y(), 0);

	::glDisable(GL_LIGHTING);

	//////////////////////////////////////////////////////////////////////////

	std::list< std::list<Point_2> >::const_iterator polygon_cur = m_polygons.begin();
	std::list< std::list<Point_2> >::const_iterator polygon_end = m_polygons.end();

	//////////////////////////////////////////////////////////////////////////
	// ƶ
	::glLineWidth(1.5f);
	for (; polygon_cur!=polygon_end; ++polygon_cur)
	{
		std::list<Point_2>::const_iterator point_cur = polygon_cur->begin();  
		std::list<Point_2>::const_iterator point_end = polygon_cur->end();
		
		//::glColor3ub(255, 0, 255);
		CGAL::Orientation o = CGAL::orientation_2(point_cur, point_end);

		// ע⣺STLģͰΧΪ⻷⻷ǡ÷
		if (o == CGAL::COUNTERCLOCKWISE)
			::glColor3ub(0, 255, 0);	// ڻɫ
		else if (o == CGAL::CLOCKWISE)  
			::glColor3ub(255, 0, 0);    // ⻷ɫ

		glBegin(GL_LINE_LOOP);
		for (; point_cur!=point_end; ++point_cur)
			glVertex2d(point_cur->x(), point_cur->y());		
		glEnd();
	}

	//////////////////////////////////////////////////////////////////////////
	// ƶж
	//::glPointSize(6.0f);
	//::glColor3ub(0, 255, 0);
	//for (polygon_cur = m_polygons.begin(); polygon_cur!=polygon_end; ++polygon_cur)
	//{
	//	std::list<Point_2>::const_iterator point_cur = polygon_cur->begin();
	//	std::list<Point_2>::const_iterator point_end = polygon_cur->end();
	//	glBegin(GL_POINTS);
	//	for (; point_cur!=point_end; ++point_cur)
	//		glVertex2d(point_cur->x(), point_cur->y());		
	//	glEnd();
	//}

	::glPopAttrib();
	::glPopMatrix();
	glEndList();
}



void PolygonSet::ComputeGeometryParameters()
{
	m_bbox = BBox_2(-FLOAT_MAX, -FLOAT_MAX, FLOAT_MAX, FLOAT_MAX);
	m_center = Point_2(0, 0);
	m_fFitScale = 1.0;

	double xMin(FLOAT_MAX);  double yMin(FLOAT_MAX);
	double xMax(-FLOAT_MAX); double yMax(-FLOAT_MAX);

	//////////////////////////////////////////////////////////////////////////

	std::list< std::list<Point_2> >::const_iterator polygon_cur = m_polygons.begin();
	std::list< std::list<Point_2> >::const_iterator polygon_end = m_polygons.end();

	for (; polygon_cur!=polygon_end; ++polygon_cur)
	{
		std::list<Point_2>::const_iterator point_cur = polygon_cur->begin();
		std::list<Point_2>::const_iterator point_end = polygon_cur->end();
		for (; point_cur!=point_end; ++point_cur)
		{
			double x = point_cur->x();
			double y = point_cur->y();

			xMin = x < xMin ? x : xMin;
			yMin = y < yMin ? y : yMin;
			xMax = x > xMax ? x : xMax;
			yMax = y > yMax ? y : yMax;
		}
	}

	m_bbox = BBox_2(xMin, yMin, xMax, yMax);

	double cx = ( xMin + xMax ) / 2.0;
	double cy = ( yMin + yMax ) / 2.0;
	m_center = Point_2(cx, cy);

	// Calculate proper rendering position
	double len = max( xMax-xMin, yMax-yMin );
	m_fFitScale = 1.5 / len;
}
