// SegmentationDoc.cpp : CSegmentationDoc ʵ
//

#include "stdafx.h"
#include "MainFrm.h"
#include "Segmentation.h"
#include "SegmentationDoc.h"
#include "SegmentationView.h"
#include "OperationView.h"
#include "TriMesh.h"

#include <fstream>


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSegmentationDoc

IMPLEMENT_DYNCREATE(CSegmentationDoc, CDocument)

BEGIN_MESSAGE_MAP(CSegmentationDoc, CDocument)
	ON_COMMAND(ID_FILE_IMPORT_CONTOURS, &CSegmentationDoc::OnFileImportContours)
	ON_COMMAND(ID_FILE_IMPORT_STL, &CSegmentationDoc::OnFileImportStl)
END_MESSAGE_MAP()


// CSegmentationDoc /

CSegmentationDoc::CSegmentationDoc()
{
	// TODO: ڴһԹ
	m_pTriMesh = NULL;
}

CSegmentationDoc::~CSegmentationDoc()
{
	delete m_pTriMesh;
}


// CSegmentationDoc 

#ifdef _DEBUG
void CSegmentationDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CSegmentationDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


CSegmentationView* CSegmentationDoc::GetDrawView() const
{
	return ((CMainFrame*)AfxGetMainWnd())->m_pDrawView;
}


COperationView* CSegmentationDoc::GetOperationView() const
{
	return ((CMainFrame*)AfxGetMainWnd())->m_pOperationView;
}


std::string CSegmentationDoc::CString2StdString(CString str) const
{
	char buf[100];

	USES_CONVERSION;
	strcpy(buf, W2A(str));

	std::string std_name = buf;

	return std_name;
}

void CSegmentationDoc::ClearAllData()
{
	if (m_pTriMesh)	{
		m_pTriMesh->Clear();
		m_pTriMesh = NULL;
	}

	m_segmentation.Clear();
	m_polygonSet.Clear();
	m_straightSkeleton.Clear();
	m_polygonDelaunayGraph.Clear();

	GetOperationView()->SetRegionDimension(0, 0);
}

void CSegmentationDoc::OnFileImportContours()
{
	// TODO: Add your command handler code here
	CString strFilter = '\0';
	strFilter += "Polygon Files(*.plgn; *.poly) |*.plgn;*.poly|";
	strFilter += "All Files(*.*) |*.*||";

	CFileDialog dlg(TRUE,  //TRUEΪ"ļ"
		NULL, 
		NULL,
		OFN_HIDEREADONLY |   //Read Only
		OFN_OVERWRITEPROMPT,  
		LPCTSTR(strFilter));

	if (IDOK == dlg.DoModal())
	{
		ClearAllData();
		std::string file_name = CString2StdString(dlg.GetPathName());
		ReadPolygonData(file_name);

		SetWindowTitle(dlg.GetFileName());
		UpdateAllViews(NULL);
	}
}

void CSegmentationDoc::OnFileImportStl()
{
	// TODO: Add your command handler code here
	CString strFilter = '\0';
	strFilter += "STL Files(*.stl; *.astl; *.bstl; *.stla; *.stlb) |*.stl;*.astl;*.bstl;*.stla;*.stlb|";
	strFilter += "All Files(*.*) |*.*||";

	CFileDialog dlg(TRUE, NULL, NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, LPCTSTR(strFilter));

	if(IDOK == dlg.DoModal()) 
	{
		//ļչ
		CString fullPathName = dlg.GetPathName();
		int nPos = fullPathName.ReverseFind('.');
		CString extName = fullPathName.Right(fullPathName.GetLength() - nPos);
		extName.MakeLower();

		if (extName == "stl" || extName == "astl" || "bstl" || "stla" || "stlb")
		{
			ClearAllData();
			
			m_pTriMesh= new TriMesh;
			m_pTriMesh->OpenSTL(fullPathName);

			m_pTriMesh->SetShowModel(true);
			m_pTriMesh->SetShowFeatureEdges(false);
			m_segmentation.SetShowSegmentation(false);

			SetWindowTitle(dlg.GetFileName());
			GetDrawView()->InvalidateRect(NULL, FALSE);
		}
		else
			AfxMessageBox(_T("޷ʶļʽ"));
	}
}

void CSegmentationDoc::SetWindowTitle(CString title) const
{
	AfxGetMainWnd()->SetWindowText(title);
}

void CSegmentationDoc::ReadPolygonData(std::string file_name)
{
	std::ifstream infile;
	infile.open(file_name.c_str());
	if (!infile) {
		AfxMessageBox(_T("ܴļ"));
		return;
	}

	int nb_all_polygon;
	infile >> nb_all_polygon;

	for (int i=0; i<nb_all_polygon; ++i)
	{
		int nb_vertices;
		infile >> nb_vertices;

		std::list<Point_2> one_polygon;

		double x, y, dummy;
		for (int j=0; j<nb_vertices; ++j)
		{			
			infile >> x >> y >> dummy;
			one_polygon.push_back( Point_2(x, y) );

		}

		m_polygonSet.m_polygons.push_back( one_polygon );
	}

	// read additional two points: the bounding box
	double minX, minY, maxX, maxY, dummy;
	infile >> maxX >> maxY >> dummy
		   >> minX >> minY >> dummy;

	BBox_2  bbox   = BBox_2(minX, minY, maxX, maxY);
	Point_2 center = Point_2((maxX + minX) / 2.0, (maxY + minY) / 2.0);
	double len = max( maxX - minX, maxY - minY );

	//////////////////////////////////////////////////////////////////////////	
	// add bounding box polygon
	double offset = len / 10;
	std::list< Point_2 >  polygon_box;
	polygon_box.push_back( Point_2(maxX + offset, maxY + offset) );
	polygon_box.push_back( Point_2(maxX + offset, minY - offset) );
	polygon_box.push_back( Point_2(minX - offset, minY - offset) );
	polygon_box.push_back( Point_2(minX - offset, maxY + offset) );

	m_polygonSet.m_polygons.push_front( polygon_box );
	m_polygonSet.ComputeGeometryParameters();
	m_polygonSet.BuildDisplayListObject();

	GetOperationView()->SetRegionDimension(maxX - minX, maxY - minY);
	GetDrawView()->InvalidateRect(NULL, FALSE);
}