#include "StdAfx.h"
#include "StraightSkeleton.h"
#include "RegionExtracter.h"


StraightSkeleton::StraightSkeleton(void)
: m_bShowContourSkeleton(true)
, m_bShowInnerSkeleton(true)
, m_bShowAllOffset(true)
, m_nSpecifiedOffsetToShow(0)
{
}

StraightSkeleton::~StraightSkeleton(void)
{
}

void StraightSkeleton::SetShowInnerSkeleton(bool b)
{
	m_bShowInnerSkeleton = b;
}

void StraightSkeleton::SetShowContourSkeleton(bool b)
{
	m_bShowContourSkeleton = b;
}

void StraightSkeleton::SetShowAllOffset(bool b)
{
	m_bShowAllOffset = b;
}

void StraightSkeleton::SetShowSpecifiedOffset(int whichOffset)
{
	m_nSpecifiedOffsetToShow = whichOffset;
}

void StraightSkeleton::Clear()
{
	m_inputRegions.clear();
	m_outputRegions.clear();
	m_offsets.clear();
	m_SSkels.clear();

	glDeleteLists(m_nDisplayListObject, 1);
	m_nDisplayListObject = 0;
}


void StraightSkeleton::BuildDisplayListObject()
{

}

void StraightSkeleton::Draw() 
{
	::glPushMatrix();
	::glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_POINT_BIT | GL_LINE_BIT | GL_LIGHTING_BIT);

	::glScalef(m_fFitScale, m_fFitScale, m_fFitScale);
	::glTranslated(-m_center.x(), -m_center.y(), 0);

	::glDisable(GL_LIGHTING);
	::glLineWidth(1.5f);

	//////////////////////////////////////////////////////////////////////////
	if (m_SSkels.size() > 0)
		DrawSkeleton(m_bShowInnerSkeleton, m_bShowContourSkeleton);
	//////////////////////////////////////////////////////////////////////////

	if (m_bShowAllOffset)
		DrawAllOffset();
	else
		DrawSpecifiedOffset(m_nSpecifiedOffsetToShow);

	::glPopAttrib();
	::glPopMatrix();
}


void StraightSkeleton::DrawSkeleton(bool showInnerSkeleton, bool showContourSkeleton) const
{
	SSkels::const_iterator curSSkel = m_SSkels.begin();
	SSkels::const_iterator endSSkel = m_SSkels.end();
	for (; curSSkel!=endSSkel; ++curSSkel)
	{
		const Halfedge_const_handle null_halfedge ;
		const Vertex_const_handle   null_vertex ;

		SSkelPtr sskelPtr = *curSSkel;
		int watchdog_limit = sskelPtr->size_of_halfedges();

		for ( Face_const_iterator fit = sskelPtr->faces_begin(), efit = sskelPtr->faces_end()
			; fit != efit
			; ++ fit)
		{
			Halfedge_const_handle hstart = fit->halfedge();
			Halfedge_const_handle he     = hstart ;

			int watchdog = watchdog_limit ;

			glBegin(GL_LINES); 
			do {
				if ( he == null_halfedge )
					break ;

				if ( he->is_bisector() )
				{
					bool lVertexOK      = he->vertex() != null_vertex ;
					bool lOppositeOK    = he->opposite() != null_halfedge ;
					bool lOppVertexOK   = lOppositeOK && he->opposite()->vertex() != null_vertex ;
					bool lVertexHeOK    = lVertexOK && he->vertex()->halfedge() != null_halfedge ;
					bool lOppVertexHeOK = lOppVertexOK && he->opposite()->vertex()->halfedge() != null_halfedge ;

					if ( lVertexOK && lOppVertexOK && lVertexHeOK && lOppVertexHeOK )
					{
						const Point_2 p0 = he->opposite()->vertex()->point();
						const Point_2 p1 = he->vertex()->point();

						if (he->is_inner_bisector() ) 
						{
							if ( showInnerSkeleton ) {
								glLineWidth(2.5f);
								glColor3f(255, 255, 0);	
								//glColor3ub(0, 0, 255);
								glVertex2d( p0.x(), p0.y() );	
								glVertex2d( p1.x(), p1.y() );	
							}
						}
						else if (showContourSkeleton)
						{
							glLineWidth(1.0f);
							glColor3ub(0, 255, 255);
							//glColor3ub(0, 0, 255);
							glVertex2d( p0.x(), p0.y() );	
							glVertex2d( p1.x(), p1.y() );	
						}
					}
				}
				he = he->next();
			} while ( --watchdog > 0 && he != hstart ) ;
			glEnd();
		}
	}  // for each sskel
}

void StraightSkeleton::CreateSkeleton()
{
	if ( m_inputRegions.empty() )
		return;

	Regions::iterator curRegion = m_inputRegions.begin();
	Regions::iterator endRegion = m_inputRegions.end();
	for (; curRegion!=endRegion; ++curRegion)
	{
		RegionPtr lRegion = *curRegion;

		SSkelBuilder builder;
		for( Region::const_iterator bit = lRegion->begin(), ebit = lRegion->end() ; bit != ebit ; ++ bit )
			builder.enter_contour((*bit)->begin(), (*bit)->end());

		SSkelPtr sskelPtr = builder.construct_skeleton() ;
		bool isValid = sskelPtr;
		if ( isValid ) 
			m_SSkels.push_back(sskelPtr);
		else
		{
			AfxMessageBox(_T("Straight Skeleton construction failed."));
			break;
		}
	}
}


void StraightSkeleton::CreateOffset(double offset)
{
	if (m_SSkels.empty()) {
		AfxMessageBox(_T("ȼǼ!"));
		return;
	}

	//AfxMessageBox(_T("ÿΧУƫ"));

	double w = m_bbox.xmax() - m_bbox.xmin();
	double h = m_bbox.ymax() - m_bbox.ymin();
	double len = min(w, h);
	if (offset <=0 || offset >= len) {
		AfxMessageBox(_T("ЧĲ!"));
		return;
	}

	// ƫþ
	Region const& lRegion = *m_inputRegions.front();
	if (lRegion.size() > 0)	{
		m_offsets.clear();
		PolygonPtr polygonPtr = lRegion[0];		
		if (polygonPtr->size() >= 3) {
			int num = ceil(len / (offset * 2.0) );
			for ( int c=1 ; c<num; ++c )
				m_offsets.insert( c*offset );
		}
	}

	// ƫ	
	m_outputRegions.clear();

	//////////////////////////////////////////////////////////////////////////

	// ÿregionʾͬƫþƫϣЩƫԲͬregion
	for ( Doubles::const_iterator i = m_offsets.begin() ; i != m_offsets.end() ; ++ i )
	{
		double offset = *i ;
		RegionPtr region( new Region );  // ÿһStraightSkeletonӦһRegionһRegion

		SSkels::const_iterator curSSkel = m_SSkels.begin();
		SSkels::const_iterator endSSkel = m_SSkels.end();
		for (; curSSkel!=endSSkel; ++curSSkel)  
		{
			SSkelPtr sskelPtr = *curSSkel;

			OffsetBuilder lOffsetBuilder(*sskelPtr);
			lOffsetBuilder.construct_offset_contours(offset, std::back_inserter(*region) );
		}	

		if ( region->size() > 0 )
			m_outputRegions.push_back(region);
	}
}

void StraightSkeleton::DrawClosedPolygon(const PolygonPtr plgn) const
{
	Polygon2::const_iterator first = plgn->begin();
	Polygon2::const_iterator end   = plgn->end();
	Polygon2::const_iterator last  = end - 1 ;
	for ( Polygon2::const_iterator it = first ; it != end ; ++ it )
	{
		Polygon2::const_iterator nx = ( it != last ? it + 1 : first ) ;

		Point_2 p0 = *it;
		Point_2 p1 = *nx;

		glVertex2d(p0.x(), p0.y());
		glVertex2d(p1.x(), p1.y());	
	}	
}

void StraightSkeleton::DrawAllOffset()
{
	glLineWidth(1.0f);
	glColor3ub(255, 255, 255);
	glBegin(GL_LINES);

	for ( Regions::const_iterator rit = m_outputRegions.begin(), erit = m_outputRegions.end(); rit != erit; ++ rit )
	{
		for ( Region::const_iterator bit = (*rit)->begin(), ebit = (*rit)->end(); bit != ebit; ++ bit )
		{
			DrawClosedPolygon(*bit);
		}		
	}
	glEnd();
}

void StraightSkeleton::DrawSpecifiedOffset(int whichOffset)
{
	glLineWidth(1.0f);
	glColor3ub(255, 255, 255);
	glBegin(GL_LINES);
		
	int i = 0;
	for ( Regions::const_iterator rit = m_outputRegions.begin(), erit = m_outputRegions.end(); rit != erit; ++rit )
	{
		if ( i == whichOffset )
		for ( Region::const_iterator bit = (*rit)->begin(), ebit = (*rit)->end(); bit != ebit; ++bit )
		{
			DrawClosedPolygon(*bit);		
		}
		++i;
	}
	glEnd();
}



void StraightSkeleton::SetPolygonData(const std::list<std::list<Point_2>>& polygons)
{
	if (polygons.empty())
		return;

	Clear();

	// convert to CGAL polygons	
	{
		//std::list< std::list<Point_2> >::const_iterator icur = polygons.begin();
		//std::list< std::list<Point_2> >::const_iterator iend = polygons.end();

		//RegionPtr lRegion( new Region() );
		//for (; icur!=iend; ++icur)
		//	lRegion->push_back( PolygonPtr( new Polygon2(icur->rbegin(), icur->rend()) ) ) ;

		//m_inputRegions.push_back( lRegion );

		//////////////////////////////////////////////////////////////////////////

		RegionExtracter re;
		re.ExtractRegions(polygons, m_inputRegions);
	}

	ComputeGeometryParameters(polygons);
}
