#ifndef _STRAIGHT_SKELETON_H_
#define _STRAIGHT_SKELETON_H_


#include "CGAL_Object.h"
#include <CGAL/Polygon_offset_builder_2.h>
#include <CGAL/Straight_skeleton_builder_2.h>
#include <CGAL/compute_outer_frame_margin.h>

#include <boost/shared_ptr.hpp>
#include <vector>
#include <list>


typedef CGAL::Segment_2<K>														Segment2;

typedef std::vector<Point_2>													Polygon2;
typedef boost::shared_ptr<Polygon2>												PolygonPtr;

typedef std::vector<PolygonPtr>													Region ;
typedef boost::shared_ptr<Region>												RegionPtr ;

typedef std::vector<RegionPtr>													Regions ;
typedef std::set<double>														Doubles ;


//////////////////////////////////////////////////////////////////////////


typedef CGAL::Straight_skeleton_2<K>											SSkel;
typedef boost::shared_ptr<SSkel>												SSkelPtr;
typedef std::vector<SSkelPtr>													SSkels;

typedef SSkel::Halfedge_iterator												Halfedge_iterator;
typedef SSkel::Vertex_handle													Vertex_handle;
typedef SSkel::Face_const_iterator												Face_const_iterator;
typedef SSkel::Halfedge_const_handle											Halfedge_const_handle ;
typedef SSkel::Vertex_const_handle												Vertex_const_handle ;


typedef CGAL::Straight_skeleton_builder_traits_2<K>								SSkelBuilderTraits;
typedef CGAL::Straight_skeleton_builder_2<SSkelBuilderTraits, SSkel>			SSkelBuilder;

typedef CGAL::Polygon_offset_builder_traits_2<K>								OffsetBuilderTraits;
typedef CGAL::Polygon_offset_builder_2<SSkel, OffsetBuilderTraits, Polygon2>	OffsetBuilder;



//////////////////////////////////////////////////////////////////////////


class StraightSkeleton : public CGAL_Object
{
public:
	StraightSkeleton(void);
	~StraightSkeleton(void);
	
	virtual void Draw();
	virtual void BuildDisplayListObject();
	virtual void Clear();

	void  SetShowInnerSkeleton(bool b);
	void  SetShowContourSkeleton(bool b);
	void  SetShowAllOffset(bool b);
	void  SetShowSpecifiedOffset(int whichOffset);

	void  SetPolygonData(const std::list<std::list<Point_2>>& polygons);
	void  CreateSkeleton();  // create inner skeleton

	void  CreateOffset(double offset);
	void  DrawAllOffset();
	void  DrawSpecifiedOffset(int whichOffset);


private:
	void  DrawSkeleton(bool showInnerSkeleton, bool showContourSkeleton) const;

	void  DrawClosedPolygon(const PolygonPtr plgn) const;

public:
	SSkels	 m_SSkels;
//	SSkelPtr m_SSkelPtr;
	Regions  m_inputRegions ;
	Regions  m_outputRegions ;

	Doubles  m_offsets ;

	bool	 m_bShowInnerSkeleton;
	bool	 m_bShowContourSkeleton;

	bool     m_bShowAllOffset;
	int      m_nSpecifiedOffsetToShow;
};


#endif