/*
* Copyright (c) 2007, Shenyang Institute of Automation, Chinese Academy
* of Sciences. , all rights reserved.
* 
* Filename     : Triangle.cpp
* Project	   : xMesh library
* Abstract     : Implementation of the Triangle class
* Version      : 1.0
* Author			: , nanlliang@sia.cn
* Original Author   : , nanlliang@sia.cn
* Accomplished date : Dec. 29, 2007
*/

//-----------------------------------------------------------------
#include "StdAfx.h"
#include "Triangle.h"

Triangle::Triangle(const Triangle &tri)
{
	vertices[0] = tri.vertices[0];
	vertices[1] = tri.vertices[1];
	vertices[2] = tri.vertices[2];
	normal = tri.normal;
}

Triangle::Triangle(const Point3D& v0, const Point3D& v1, const Point3D& v2)
{
	vertices[0] = v0;
	vertices[0] = v0;
	vertices[1] = v1;
	vertices[2] = v2;
	normal = Vector3D(0, 0, 0);
}

Triangle::Triangle(const Point3D& v0, const Point3D& v1, const Point3D& v2, const Vector3D& nor)
{
	vertices[0] = v0;
	vertices[1] = v1;
	vertices[2] = v2;
	normal = nor;
}


const Triangle& Triangle::operator =(const Triangle& tri)
{
	normal = tri.normal;
	for(int i=0;i<3;i++)
		vertices[i] = tri.vertices[i];
	return *this;
}


bool Triangle::operator <(const Triangle& tri) const
{
	return 
		( vertices[0] < tri.vertices[0] ) ||
		(( vertices[0] == tri.vertices[0]) && (vertices[1] < tri.vertices[1]) ) || 
		(( vertices[0] == tri.vertices[0]) && (vertices[1] == tri.vertices[1]) && (vertices[2] < tri.vertices[2]));		
}


bool Triangle::operator ==(const Triangle& tri) const
{
	return	
		((vertices[0] == tri.vertices[0]) && (vertices[1] == tri.vertices[1]) && (vertices[2] == tri.vertices[2])) ||
		((vertices[0] == tri.vertices[0]) && (vertices[1] == tri.vertices[2]) && (vertices[2] == tri.vertices[1])) ||
		((vertices[0] == tri.vertices[1]) && (vertices[1] == tri.vertices[0]) && (vertices[2] == tri.vertices[2])) ||
		((vertices[0] == tri.vertices[1]) && (vertices[1] == tri.vertices[2]) && (vertices[2] == tri.vertices[0])) ||
		((vertices[0] == tri.vertices[2]) && (vertices[1] == tri.vertices[0]) && (vertices[2] == tri.vertices[1])) ||
		((vertices[0] == tri.vertices[2]) && (vertices[1] == tri.vertices[1]) && (vertices[2] == tri.vertices[0]));
}

Point3D Triangle::operator [](const int i) const
{
	switch (i)
	{
	case 0:
		return vertices[0];
	case 1:
		return vertices[1];
	case 2:
		return vertices[2];
	default:
		break;
	}
	return vertices[0];
}

Point3D& Triangle::operator [](const int i)
{
	switch (i)
	{
	case 0:
		return vertices[0];
	case 1:
		return vertices[1];
	case 2:
		return vertices[2];
	default:
		break;
	}
	return vertices[0];
}