/*
* Copyright (c) 2007, Shenyang Institute of Automation, Chinese Academy
* of Sciences. , all rights reserved.
* 
* Filename     : Vector2D.cpp
* Project	   : xMesh library
* Abstract     : Implementation of the Vector2D class
* Version      : 1.0
* Author			: , nanlliang@sia.cn
* Original Author   : , nanlliang@sia.cn
* Accomplished date : Dec. 29, 2007
*/

//-----------------------------------------------------------------


#include "StdAfx.h"
#include "Point2D.h"
#include "Vector2D.h"


Vector2D::Vector2D() : x(0), y(0)
{}

Vector2D::Vector2D(double _x, double _y) : x(_x), y(_y)
{}

Vector2D::Vector2D(double* p)
{
	x = p[0]; y = p[1];
}

Vector2D::Vector2D(const Point2D& p)
{
	x = p.GetX(); y = p.GetY();
}

Vector2D::Vector2D(const Vector2D& v)
{
	x = v.GetX(); y = v.GetY();
}

Vector2D::Vector2D(const Point2D& s, const Point2D& d)
{
	(*this) = d - s;
}

Vector2D::Vector2D(const Vector2D& s, const Vector2D& d)
{
	(*this) = d - s;
}

void Vector2D::SetValue(double _x, double _y) 
{
	x = _x; y = _y;
}

void Vector2D::SetValue(const Point2D& p)
{
	x = p.GetX(); y = p.GetY();
}

void Vector2D::SetValue(const Vector2D& v)
{
	x = v.GetX(); y = v.GetY();
}

void Vector2D::SetZero()
{
	SetValue(0, 0);
}

void Vector2D::SetX(double _x) { x = _x; }

void Vector2D::SetY(double _y) { y = _y; }

double Vector2D::GetX() const { return x; }

double Vector2D::GetY() const { return y; }

Point2D Vector2D::Point() const
{
	return Point2D(x, y);
}

void Vector2D::operator =(const Vector2D &v)
{ 
	x = v.GetX(); y = v.GetY();
}

Vector2D Vector2D::operator+(const Vector2D& v) const
{
	return Vector2D(x + v.GetX(), y + v.GetY());
}

Vector2D Vector2D::operator-(const Vector2D& v) const
{
	return Vector2D(x - v.GetX(), y - v.GetY());
}

Vector2D Vector2D::operator*(const double s) const
{
	return Vector2D(x * s, y * s);
}

Vector2D Vector2D::operator/(const double s) const
{
	return Point2D(x / s, y / s);
}

void Vector2D::operator+=(const Vector2D& v)
{
	x += v.GetX(); y += v.GetY();
}

void Vector2D::operator-=(const Vector2D& v)
{
	x -= v.GetX(); y -= v.GetY();
}

void Vector2D::operator*=(const double s)
{
	(*this) = (*this) * s;
}

void Vector2D::operator/=(const double s)
{
	(*this) = (*this) * s;
}

Vector2D Vector2D::operator-()
{
	return Vector2D(-x, -y);
}

double Vector2D::operator [] (const int i) const
{
	switch (i)
	{
	case 0:	return x;
	case 1:	return y;
	default: break;
	}
	return x;
}

double& Vector2D::operator [] (const int i)
{
	switch (i)
	{
	case 0:	return x;
	case 1:	return y;
	default: break;
	}
	return x;
}

bool Vector2D::operator==(Vector2D v) const
{
	return (x == v.GetX()) && (y == v.GetY());
}


//dot product
double Vector2D::DotProduct(Vector2D v) const
{
	return x * v.GetX() + y * v.GetY(); 
}

void Vector2D::Normalize()
{
	double len = Length();
	if (len != 0)
	{
		x /= len; y /= len;
	}
	else
	{
		x = 0; y = 0;
	}
}

//length
double Vector2D::Length() const
{ 
	return sqrt(x*x + y*y);
}

