/*
* Copyright (c) 2007, Shenyang Institute of Automation, Chinese Academy
* of Sciences. , all rights reserved.
* 
* Filename     : Vector2D.h
* Project	   : xMesh library
* Abstract     : Interface for the Vector2D class.
* Version      : 1.0
* Author			: , nanlliang@sia.cn
* Original Author   : , nanlliang@sia.cn
* Accomplished date : Dec. 29, 2007
*/

//-----------------------------------------------------------------

#pragma once
#include <cmath>

class Point2D;

// Vector2D class
class Vector2D
{
public:
	Vector2D();
	Vector2D(double _x, double _y);
	Vector2D(double* p);
	Vector2D(const Point2D& p);
	Vector2D(const Vector2D& v);
	Vector2D(const Point2D& s, const Point2D& d);
	Vector2D(const Vector2D& s, const Vector2D& d);
	~Vector2D() {};

	void   SetValue(double _x, double _y);
	void   SetValue(const Point2D& p);
	void   SetValue(const Vector2D& v);
	void   SetZero();
	void   SetX(double _x);
	void   SetY(double _y);
	double GetX() const;
	double GetY() const;

	Point2D Point() const;

	void operator =(const Vector2D &v);

	Vector2D operator+(const Vector2D& v) const;
	Vector2D operator-(const Vector2D& v) const;
	Vector2D operator*(const double s) const;
	Vector2D operator/(const double s) const;

	void operator+=(const Vector2D& v);
	void operator-=(const Vector2D& v);
	void operator*=(const double s);
	void operator/=(const double s);

	Vector2D operator-();

	double  operator [] (const int i) const;
	double& operator [] (const int i);

	bool operator==(Vector2D v) const;

	//dot product
	double DotProduct(Vector2D v) const;

	void   Normalize();
	double Length() const;

public:
	double x;
	double y;
};
