/*
* Copyright (c) 2007, Shenyang Institute of Automation, Chinese Academy
* of Sciences. , all rights reserved.
* 
* Filename     : Vector3D.cpp
* Project	   : xMesh library
* Abstract     : Implementation of the Vector3D class
* Version      : 1.0
* Author			: , nanlliang@sia.cn
* Original Author   : , nanlliang@sia.cn
* Accomplished date : Dec. 29, 2007
*/

//-----------------------------------------------------------------


#include "StdAfx.h"
#include "Point3D.h"
#include "Vector3D.h"


Vector3D::Vector3D() : x(0), y(0), z(0)
{}

Vector3D::Vector3D(double _x, double _y, double _z) : x(_x), y(_y), z(_z) 
{}

Vector3D::Vector3D(double* p)
{
	x = p[0]; y = p[1]; z = p[2];
}

Vector3D::Vector3D(const Point3D& p)
{
	x = p.GetX(); y = p.GetY(); z = p.GetZ(); 
}

Vector3D::Vector3D(const Vector3D& v)
{
	x = v.GetX(); y = v.GetY(); z = v.GetZ(); 
}

Vector3D::Vector3D(const Point3D& s, const Point3D& d)
{
	(*this) = d - s;
}

Vector3D::Vector3D(const Vector3D& s, const Vector3D& d)
{
	(*this) = d - s;
}

void Vector3D::SetValue(double _x, double _y, double _z) 
{
	x = _x; y = _y; z = _z;
}

void Vector3D::SetValue(const Point3D& p)
{
	x = p.GetX(); y = p.GetY(); z = p.GetZ(); 
}

void Vector3D::SetValue(const Vector3D& v)
{
	x = v.GetX(); y = v.GetY(); z = v.GetZ(); 
}

void Vector3D::SetZero()
{
	SetValue(0, 0, 0);
}

void Vector3D::SetX(double _x) { x = _x; }

void Vector3D::SetY(double _y) { y = _y; }

void Vector3D::SetZ(double _z) { z = _z; }

double Vector3D::GetX() const { return x; }

double Vector3D::GetY() const { return y; }

double Vector3D::GetZ() const { return z; }

Point3D Vector3D::Point() const
{
	return Point3D(x, y, z);
}

void Vector3D::operator =(const Vector3D &v)
{ 
	x = v.GetX(); y = v.GetY(); z = v.GetZ(); 
}

Vector3D Vector3D::operator+(const Vector3D& v) const
{
	return Vector3D(x + v.GetX(), y + v.GetY(), z + v.GetZ());
}

Vector3D Vector3D::operator-(const Vector3D& v) const
{
	return Vector3D(x - v.GetX(), y - v.GetY(), z - v.GetZ());
}

Vector3D Vector3D::operator*(const double s) const
{
	return Vector3D(x * s, y * s, z * s);
}

Vector3D Vector3D::operator/(const double s) const
{
	return Point3D(x / s, y / s, z / s);
}

void Vector3D::operator+=(const Vector3D& v)
{
	x += v.GetX(); y += v.GetY(); z += v.GetZ(); 
}

void Vector3D::operator-=(const Vector3D& v)
{
	x -= v.GetX(); y -= v.GetY(); z -= v.GetZ(); 
}

void Vector3D::operator*=(const double s)
{
	(*this) = (*this) * s;
}

void Vector3D::operator/=(const double s)
{
	(*this) = (*this) * s;
}

Vector3D Vector3D::operator-()
{
	return Vector3D(-x, -y, -z);
}

double Vector3D::operator [] (const int i) const
{
	switch (i)
	{
	case 0:	return x;
	case 1:	return y;
	case 2:	return z;
	default: break;
	}
	return x;
}

double& Vector3D::operator [] (const int i)
{
	switch (i)
	{
	case 0:	return x;
	case 1:	return y;
	case 2:	return z;
	default: break;
	}
	return x;
}

bool Vector3D::operator==(Vector3D v) const
{
	return (x == v.GetX()) && (y == v.GetY()) && (z == v.GetZ());
}

//cross product
Vector3D Vector3D::operator *(Vector3D v) const
{
	return Vector3D(y * v.GetZ() - z * v.GetY(), 
		            z * v.GetX() - x * v.GetZ(),
		            x * v.GetY() - y * v.GetX());
}

Vector3D Vector3D::CrossProduct(Vector3D v) const 
{
	return (*this) * v;
}

//dot product
double Vector3D::DotProduct(Vector3D v) const
{
	return x * v.GetX() + y * v.GetY() + z * v.GetZ(); 
}

void Vector3D::Normalize()
{
	double len = Length();
	if (len != 0)
	{
		x /= len; y /= len;	z /= len;
	}
	else
	{
		x = 0; y = 0; z = 0;	
	}
}

//length
double Vector3D::Length() const
{ 
	return sqrt(x*x + y*y + z*z);
}

