
/*
* Copyright (c) 2007, Shenyang Institute of Automation, Chinese Academy
* of Sciences. , all rights reserved.
* 
* Filename     : Vector3D.h
* Project	   : xMesh library
* Abstract     : Interface for the Vector3D class.
* Version      : 1.0
* Author			: , nanlliang@sia.cn
* Original Author   : , nanlliang@sia.cn
* Accomplished date : Dec. 29, 2007
*/

//-----------------------------------------------------------------
#ifndef _VECTOR3D_H_
#define _VECTOR3D_H_


#include <cmath>

class Point3D;

// Vector3D class
class Vector3D
{
public:
	Vector3D();
	Vector3D(double _x, double _y, double _z);
	Vector3D(double *p);
	Vector3D(const Point3D& p);
	Vector3D(const Vector3D& v);
	Vector3D(const Point3D& s, const Point3D& d);
	Vector3D(const Vector3D& s, const Vector3D& d);
	~Vector3D() {};

	void   SetValue(double _x, double _y, double _z);
	void   SetValue(const Point3D& p);
	void   SetValue(const Vector3D& v);
	void   SetZero();
	void   SetX(double _x);
	void   SetY(double _y);
	void   SetZ(double _z);
	double GetX() const;
	double GetY() const;
	double GetZ() const;

	Point3D Point() const;

	void operator =(const Vector3D &v);

	Vector3D operator+(const Vector3D& v) const;
	Vector3D operator-(const Vector3D& v) const;
	Vector3D operator*(const double s) const;
	Vector3D operator/(const double s) const;

	void operator+=(const Vector3D& v);
	void operator-=(const Vector3D& v);
	void operator*=(const double s);
	void operator/=(const double s);

	Vector3D operator-();

	double  operator [] (const int i) const;
	double& operator [] (const int i);

	bool operator==(Vector3D v) const;

	//cross product
	Vector3D operator*(Vector3D v) const;
	Vector3D CrossProduct(Vector3D v) const;

	//dot product
	double   DotProduct(Vector3D v) const;

	void   Normalize();
	double Length() const;

public:
	double x;
	double y;
	double z;
};

#endif