
#ifndef PDG_INSERT_H
#define PDG_INSERT_H


int get_new_id()
{
	static int gen = 0;
	gen++;

	assert( gen != 0 ); // guards against unwanted phenomena due to
	// overflow
	return gen;
}


template<class PDG, class PointT>
typename PDG::Vertex_handle
insert_segment(PDG& pdg, const PointT& p1, const PointT& p2, int id)
{
	typedef typename PDG::Vertex_handle Vertex_handle;

	Vertex_handle v1 = pdg.insert(p1);
	Vertex_handle v2 = pdg.insert(p2);
	Vertex_handle v3 = pdg.insert(p1, p2);

	if ( v3 == Vertex_handle() )
		return v3;

	v1->set_info( id );
	v2->set_info( id );
	v3->set_info( id );
	return v3;
}

template<class PDG, class Plgn>
typename PDG::Vertex_handle
insert_polygon(PDG& pdg, const Plgn& pgn)
{
	typedef typename PDG::Vertex_handle Vertex_handle;

	int id = get_new_id();

	Vertex_handle v;
	int psize = pgn.size();
	for (int i = 0; i < psize; i++ )
	{
		v = insert_segment( pdg, pgn[i], pgn[(i+1) % psize], id );

		if ( v == Vertex_handle() ) 
			break;
	}
	return v;
}



#endif // PDG_INSERT_H
